@@ -105,9 +105,11 @@ opal.file_upload <- function(opal, source, destination, all.files = TRUE) {
   
   doUploadFile <- function(src, dest) {
     location <- append("files", strsplit(substring(dest, 2), "/")[[1]])
-    r <- POST(.url(opal, location), body=list(file=upload_file(src)), encode = "multipart", 
-              content_type("multipart/form-data"), accept("text/html"), 
-              config=opal$config, handle=opal$handle, .verbose())
+    r <- httr::POST(.url(opal, location),
+                    httr::add_headers('X-XSRF-TOKEN' = opal$csrf),
+                    body=list(file=upload_file(src)), encode = "multipart", 
+                    content_type("multipart/form-data"), accept("text/html"), 
+                    config=opal$config, handle=opal$handle, .verbose())
     res <- .handleResponse(opal, r)
   }
   
